@@ -4,14 +4,17 @@ use strict;
 use warnings;
 use 5.010_000;
 
-our $VERSION = '2.028002_001';
+our $VERSION = '2.029013_001';
 use App::Netdisco::Configuration;
 
+use Module::Find ();
+Module::Find::usesub 'App::NetdiscoE::Init';
+
 =head1 NAME
 
 App::Netdisco - An open source web-based network management tool.
 
-=head1 Introduction
+=head1 DESCRIPTION
 
 Netdisco is a web-based network management tool designed for network
 administrators. Data is collected into a PostgreSQL database using SNMP.
@@ -72,7 +75,8 @@ On Fedora/Red-Hat:
 
  root:~# yum install perl-core perl-DBD-Pg net-snmp-perl make automake gcc
 
-With those installed, we can proceed...
+With those installed, next check that your system's clock is correct. Then, we
+can proceed...
 
 Create a user on your system called C<netdisco> if one does not already exist.
 We'll install Netdisco and its dependencies into this user's home area, which
@@ -187,7 +191,11 @@ Netdisco 2, disable your system's cron jobs for the Netdisco 1.x poller.
 For further documentation on deployment, see
 L<Deployment|App::Netdisco::Manual::Deployment>.
 
-=head1 Upgrading
+=head1 Upgrading from 2.x
+
+If you're running a version of Netdisco prior to 2.x then you should follow
+the full installation instructions, above. This process is for upgrading
+version 2.x only.
 
 Before upgrading please review the latest L<Release
 Notes|App::Netdisco::Manual::ReleaseNotes>. Then, the process is as follows:
