@@ -3,18 +3,20 @@ package App::Netdisco::JobQueue::PostgreSQL;
 use Dancer qw/:moose :syntax :script/;
 use Dancer::Plugin::DBIC 'schema';
 
+use App::Netdisco::Daemon::Job;
 use Net::Domain 'hostfqdn';
+use Module::Load ();
 use Try::Tiny;
 
 use base 'Exporter';
 our @EXPORT = ();
 our @EXPORT_OK = qw/
   jq_getsome
+  jq_getsomep
   jq_locked
   jq_queued
   jq_log
   jq_userlog
-  jq_take
   jq_lock
   jq_defer
   jq_complete
@@ -24,23 +26,25 @@ our @EXPORT_OK = qw/
 our %EXPORT_TAGS = ( all => \@EXPORT_OK );
 
 sub jq_getsome {
-  my $num_slots = shift;
-  my @returned = ();
+  my ($num_slots, $prio) = @_;
+  return () if ((!defined $num_slots) or ($num_slots < 1));
+  $prio ||= 'normal';
 
   my $rs = schema('netdisco')->resultset('Admin')
     ->search(
-      {status => 'queued'},
-      {order_by => 'random()', rows => ($num_slots || 1)},
+      {status => 'queued', action => { -in => setting('job_prio')->{$prio} } },
+      {order_by => 'random()', rows => $num_slots},
     );
 
+  my @returned = ();
   while (my $job = $rs->next) {
-      my $job_type = setting('job_types')->{$job->action} or next;
-      push @returned, schema('daemon')->resultset('Admin')
-        ->new_result({ $job->get_columns, type => $job_type });
+      push @returned, App::Netdisco::Daemon::Job->new({ $job->get_columns });
   }
   return @returned;
 }
 
+sub jq_getsomep { return jq_getsome(shift, 'high') }
+
 sub jq_locked {
   my $fqdn = hostfqdn || 'localhost';
   my @returned = ();
@@ -49,9 +53,7 @@ sub jq_locked {
     ->search({status => "queued-$fqdn"});
 
   while (my $job = $rs->next) {
-      my $job_type = setting('job_types')->{$job->action} or next;
-      push @returned, schema('daemon')->resultset('Admin')
-        ->new_result({ $job->get_columns, type => $job_type });
+      push @returned, App::Netdisco::Daemon::Job->new({ $job->get_columns });
   }
   return @returned;
 }
@@ -67,50 +69,18 @@ sub jq_queued {
 }
 
 sub jq_log {
-  my @returned = ();
-
-  my $rs = schema('netdisco')->resultset('Admin')->search({}, {
+  return schema('netdisco')->resultset('Admin')->search({}, {
     order_by => { -desc => [qw/entered device action/] },
     rows => 50,
-  });
-
-  while (my $job = $rs->next) {
-      my $job_type = setting('job_types')->{$job->action} or next;
-      push @returned, schema('daemon')->resultset('Admin')
-        ->new_result({ $job->get_columns, type => $job_type });
-  }
-  return @returned;
+  })->with_times->hri->all;
 }
 
 sub jq_userlog {
   my $user = shift;
-  my @returned = ();
-
-  my $rs = schema('netdisco')->resultset('Admin')->search({
+  return schema('netdisco')->resultset('Admin')->search({
     username => $user,
     finished => { '>' => \"(now() - interval '5 seconds')" },
-  });
-
-  while (my $job = $rs->next) {
-      my $job_type = setting('job_types')->{$job->action} or next;
-      push @returned, schema('daemon')->resultset('Admin')
-        ->new_result({ $job->get_columns, type => $job_type });
-  }
-  return @returned;
-}
-
-# PostgreSQL engine depends on LocalQueue, which is accessed synchronously via
-# the main daemon process. This is only used by daemon workers which can use
-# MCE ->do() method.
-sub jq_take {
-  my ($self, $wid, $type) = @_;
-
-  # be polite to SQLite database (that is, local CPU)
-  debug "$type ($wid): sleeping now...";
-  sleep(1);
-
-  debug "$type ($wid): asking for a job";
-  $self->do('take_jobs', $wid, $type);
+  })->with_times->hri->all;
 }
 
 sub jq_lock {
@@ -122,7 +92,7 @@ sub jq_lock {
   try {
     schema('netdisco')->txn_do(sub {
       schema('netdisco')->resultset('Admin')
-        ->find($job->id, {for => 'update'})
+        ->find($job->job, {for => 'update'})
         ->update({ status => "queued-$fqdn" });
 
       # remove any duplicate jobs, needed because we have race conditions
@@ -136,6 +106,9 @@ sub jq_lock {
       }, {for => 'update'})->delete();
     });
     $happy = true;
+  }
+  catch {
+    error $_;
   };
 
   return $happy;
@@ -145,14 +118,17 @@ sub jq_defer {
   my $job = shift;
   my $happy = false;
 
-  # lock db row and update to show job is available
   try {
+    # lock db row and update to show job is available
     schema('netdisco')->txn_do(sub {
       schema('netdisco')->resultset('Admin')
-        ->find($job->id, {for => 'update'})
-        ->update({ status => 'queued' });
+        ->find($job->job, {for => 'update'})
+        ->update({ status => 'queued', started => undef });
     });
     $happy = true;
+  }
+  catch {
+    error $_;
   };
 
   return $happy;
@@ -166,13 +142,17 @@ sub jq_complete {
   try {
     schema('netdisco')->txn_do(sub {
       schema('netdisco')->resultset('Admin')
-        ->find($job->id, {for => 'update'})->update({
+        ->find($job->job, {for => 'update'})->update({
           status => $job->status,
           log    => $job->log,
+          started  => $job->started,
           finished => $job->finished,
         });
     });
     $happy = true;
+  }
+  catch {
+    error $_;
   };
 
   return $happy;
@@ -198,6 +178,9 @@ sub jq_insert {
       ]);
     });
     $happy = true;
+  }
+  catch {
+    error $_;
   };
 
   return $happy;
